#!/usr/bin/env python
#
# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2016 USIT-University of Oslo
#
# zabbix_get_webapps_info.py is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either version
# 3 of the License, or (at your option) any later version.
#
# zabbix_get_webapps_info.py is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with sms_send.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import glob
import json

# ############################################
# Method generate_webapps_info_file()
# ############################################

def generate_webapps_info_file(src_directory,output_directory):
    """
    Generate webapps info file with metadata and components
    information for all web applications on the server. 
    """

    try:

        #
        # Open the output file that will have the webapp information
        # that will be needed to generate the zabbix autodiscovery
        # regel. This file will get indexed by Nivlheim for post
        # processing.
        #
        with open (output_directory + '/web-applications.txt','w') as output_file:

            #
            # Get all status json files delivered by the web aplications.
            # Format: <src_directory>/<application-name>-health.json
            #
            src_files = glob.glob(src_directory + '/*-health.json')

            for src_file in src_files:
            
                #
                # Open every webapp status file that exist in this
                # server, and read the json data
                #
                # Continue the execution with the next json file if
                # one of the files can not be open or has not json
                # code.
                #
                try:
                    with open(src_file,'r') as info_file:    
                        data = json.load(info_file)

                except Exception,e:
                    print e
                    continue
                    pass
                
                monitor_keys = []

                #
                # Json data must have all metadata attributes defined
                # to be considered.
                #

                if len(set(['zabbix-name','instance','host-group','url','updated']) - set(data.keys())) == 0:

                    #
                    # Create a list with all non-metadata
                    # attributes. They will be the components needed
                    # by the application. We will use this data to
                    # generate the output needed by the zabbix
                    # autodiscovery rule used to discover components.
                    #
                    for key in data.keys():
                        if key not in ['zabbix-name','instance','host-group','url','updated']:
                            monitor_keys.append(key)

                    #
                    # Save webapp information in the output file
                    #
                    monitor_keys_output = '::'.join(monitor_keys)
                    output_file.write(data["zabbix-name"] + '::' + data['instance'] + '::' + data['host-group'] + '::' + data['url'] + '::' + monitor_keys_output + '\n')

    except Exception,e:
        print e
        sys.exit(1)


# ############################################
# Main
# ############################################

if __name__ == '__main__':

    try:

        if len(sys.argv) == 3:
            src_directory = sys.argv[1]
            output_directory = sys.argv[2]

            generate_webapps_info_file(src_directory,output_directory)

        else:
            print "Error: Wrong number of parameters"
            print 'Format: ' + sys.argv[0] + ' <source_directory> <output_directory>'        
            sys.exit(1)

    except Exception,e:
        print e
        sys.exit(1)
