#!/usr/bin/env python
#
# Authors:
# rafael@postgresql.org.es / http://www.postgresql.org.es/
#
# Copyright (c) 2016 USIT-University of Oslo
#
# get_webapps_get_status.py is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either version
# 3 of the License, or (at your option) any later version.
#
# get_webapps_get_status.py is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with sms_send.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import sys
import glob
import json
import tempfile

# Zabbix proxies
zabbix_proxy = ['zabbix-proxy-prod01.uio.no','zabbix-proxy-prod02.uio.no']

# Path to zabbix_sender
zabbix_sender = '/usr/bin/zabbix_sender'

# Temp file with full json output
tmp_stat_file = tempfile.NamedTemporaryFile(delete=False,dir='/tmp')


# ############################################
# generate_webapps_health_output()
# ############################################

def generate_webapps_health_output(src_directory):

    try:

        result = ''

        #
        # Get list with all webapps health files
        #
        src_files = glob.glob(src_directory + '/*-health.json')

        for src_file in src_files:
            
            #
            # Open every webapp health file that exist in this
            # server, and read the json data
            #
            with open(src_file,'r') as info_file:    
                data = json.load(info_file)

            #
            # Check if the supported attributes are present in the
            # webapp status file, if one is not present use the return
            # value -1 (not present).
            #
            # Convert also True to 1 and False to 0.
            # 
                
            supported_attributes = ['database','ldap','tsd','updated','vortex']

            for attribute in supported_attributes:

                if attribute in data.keys():
                    if data[attribute] == True:
                        data[attribute] = 1
                    elif data[attribute] == False: 
                        data[attribute] = 0
                else:
                    data[attribute] = -1
            if data["zabbix-name"] != '':

                result += data["zabbix-name"] + ' webapp.health.database[' + data["instance"] + '] ' + str(data['database']) + '\n'
                result += data["zabbix-name"] + ' webapp.health.ldap[' + data["instance"] + '] ' + str(data['ldap']) + '\n'
                result += data["zabbix-name"] + ' webapp.health.tsd[' + data["instance"] + '] ' + str(data['tsd']) + '\n'
                result += data["zabbix-name"] + ' webapp.health.lastupdate[' + data["instance"] + '] ' + str(data['updated']) + '\n'
                result += data["zabbix-name"] + ' webapp.health.vortex[' + data["instance"] + '] ' + str(data['vortex']) + '\n'

        return result

    except Exception,e:
        raise Exception(str(e)) 


# ############################################
# Main
# ############################################

if __name__ == '__main__':

    try:

        if len(sys.argv) == 2:

            src_directory = sys.argv[1]

            result = generate_webapps_health_output(src_directory)

            #
            # We create a file with the data that zabbix_sender will
            # send in a bulk execution.
            #

            with open(tmp_stat_file.name,'w') as f: 
                f.write(result)
               
            #
            # The monitoring of this host can be done by any of the
            # zabbix proxyer defined in zabbix_proxy[]. We try all of
            # them until one of them accepts our data
            #
            
            for proxy in zabbix_proxy:
                returncode = os.system(zabbix_sender + ' -z ' + proxy + ' -i ' + tmp_stat_file.name + ' > /dev/null 2>&1')
                                
                if returncode == 0:
                    break

        else:
            print "1"
            sys.exit(1)

    except Exception, e:
        print "1"
        sys.exit(1)

    # Delete temp file with zabbix_sender data
    os.remove(tmp_stat_file.name)

    # Return value 0 = execution OK
    print "0"
