#!/usr/bin/env python
#
# Authors:
# Mustafa Ocak
# muo@uio.no
#
# Copyright (c) 2014 USIT-University of Oslo

import json
import re
import shlex
import subprocess
import sys

def run_teamdctl(teamname):
    '''
    Running teamdctl <teamname> state dump
    '''
    command = "teamdctl "+teamname+" state dump"

    clicmd=shlex.split(command)
    #print "running "+command+"."
    ps= subprocess.Popen(clicmd,shell=False,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    out= ps.communicate()
    if out[0] != "":
        jsonout = json.loads(out[0])

        return jsonout
    else:
        raise ValueError(out[1].strip('\n'))

def get_team_list():
    '''
    Find number of network teaming defined.
    '''

    command = "nmcli -p d"
    team=[]
    clicmd=shlex.split(command)

    ps= subprocess.Popen(clicmd,shell=False,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    out= ps.communicate()
    
    lines = out[0].splitlines()
    for line in lines:
        matchobj = re.search(r'team',line,re.IGNORECASE)
        if matchobj:
            splline = line.split()
            team.append(splline[0])
    
    if len(team)>0:
        return team
    else:
        raise ValueError("Master team interface not found!!")



def get_interface_status(jsonout,interface):
    '''
    returns
    0 for up
    1 for down
    2 for interface not teamed
    '''
    # when all the ports is down,  jsonout.keys() doesn't have 'ports' key.
    if ("ports" in jsonout.keys()):
   
        if (interface in jsonout['ports'].keys()):
            if jsonout['ports'][interface]['link_watches']['up']:
                return 0
            else:
                return 1
        else:
            return 2
    else: # all the teamed interfaces are down
        return 1

def get_team_status(jsonout):
    '''
    returns
    0 for up
    1 for down
    3 for degraded. (if there is only one interface in team, then status is sat to 3)
    '''
    activeinterface = jsonout['runner']['active_port']
    status = 1
    degraded = False
    # when all the ports is down jsonout['runner']['active_port']==""
    if (activeinterface != ""):
        if jsonout['ports'][activeinterface]['link_watches']['up']:
    #        for interface in jsonout['ports'].keys():
            if ( len(jsonout['ports'].keys())<2):
                #if not jsonout['ports'][interface]['link_watches']['up']:
                degraded = True
            if degraded:
                status=3
            else:
                status=0
        else:
            status=1
    else: # when all the interfaces are down
        status=1
        
    return status

def get_team_activeport(jsonut):
    activeinterface = jsonout['runner']['active_port']
    return activeinterface

def get_team_ports(jsonout):
    '''
    Find ports (network interface) defined for a given network teaming
    '''

    ports=[]

    # when all the ports is down,  jsonout.keys() doesn't have 'ports' key.
    if ("ports" in jsonout.keys()):
        ports = jsonout['ports'].keys()
        return ports
    else: # all the ports is down.
        raise ValueError ("All the slave port interfaces are down!!!")
        


def print_usage():

    print "Error: Wrong number of parameters"
    print 'Format: ' + sys.argv[0] + ' <teamname> <interfacename> status'
    print 'Format: ' + sys.argv[0] + ' <teamname> status|activeport'
    print 'Format: ' + sys.argv[0] + ' team|port|testdiscovery'

# ############################################
# MAIN
# ############################################

if __name__ == '__main__':

    teamed=[]

    zabbixjson = {}
    data=[]
    teamdic = {}

    try:

        if len(sys.argv) == 4:

            teamname=sys.argv[1].lower()
            interface=sys.argv[2].lower()
            item=sys.argv[3].lower()

            try:
                jsonout = run_teamdctl(teamname)
            except Exception as e:
                print e
                sys.exit(1)
 
            if (item =="status"):
                print get_interface_status(jsonout,interface)
            else:
                print_usage()
                sys.exit(1)

        elif len(sys.argv) == 3:
            teamname=sys.argv[1].lower()
            item=sys.argv[2].lower()
            try:
                jsonout = run_teamdctl(teamname)
            except Exception as e:
                print e
                sys.exit(1)

            if (item =="status"):
                print get_team_status(jsonout)
            elif (item == "activeport"):
                print get_team_activeport(jsonout)
            else:
                print_usage()
                sys.exit(1)
        elif len(sys.argv) == 2:
            discovertype=sys.argv[1].lower()
            if (discovertype=="team"):
                try:
                    teamed = get_team_list()
                except Exception as e:
                    print e
                    sys.exit(1)

                for team in teamed:
                    teamdic = { "{#TEAMNAME}":team}
                    data.append(teamdic)

                zabbixstr = { "data":data}
                zabbixjson = json.dumps(zabbixstr, indent=4, sort_keys=True)
                print zabbixjson

            elif (discovertype=="port"):
                try:
                    teamed = get_team_list()
                except Exception as e:
                    print e
                    sys.exit(1)

                for team in teamed:
                    jsonout = run_teamdctl(team)
                    try:
                        interfacelist = get_team_ports(jsonout)

                        for interface in interfacelist: 
                            teamdic = { "{#TEAMNAME}":team, "{#PORTIFNAME}":interface}
                            data.append(teamdic)

                    except Exception as e:
                        print e
                        sys.exit(1)
                    
                zabbixstr = { "data":data}
                zabbixjson = json.dumps(zabbixstr, indent=4, sort_keys=True)
                print zabbixjson
            elif (discovertype=="testdiscovery"):
                try:
                    teamed = get_team_list()
                    for team in teamed:
                        jsonout = run_teamdctl(team)
                        interfacelist = get_team_ports(jsonout)
                    print "Ok"
                except Exception as e:
                    print e
                    sys.exit(1)
            else:
                print_usage()
                sys.exit(1)

        else:
            print_usage()
            sys.exit(1)

    except Exception as e:
        print e
        sys.exit(1)

